﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Example3_AnalogInputs
{
    class Program
    {

        public enum pinFunctionsEnum
        {
            pinFunctionInactive = 0,
            pinFunctionDigitalInput = 2,
            pinFunctionDigitalOutput = 4,
            pinFunctionAnalogInput = 8,
            pinFunctionAnalogOutput = 16,
            pinInvert = 128
        }


        static void Main(string[] args)
        {
            Console.WriteLine("PoKeys55 Example 3 - analog inputs\n");

            // Create PoKeys55 device object
            PoKeysDevice_DLL.PoKeysDevice device = new PoKeysDevice_DLL.PoKeysDevice();

            // Enumerate devices (this step must never be ommited!)
            int iNumDevices = device.EnumerateDevices();

            if (iNumDevices == 0)
            {
                Console.WriteLine("No PoKeys55 devices found!");


                Console.WriteLine("\nPress any key to exit");
                Console.ReadKey();
                return;
            }

            // Connect to first PoKeys55 device
            device.ConnectToDevice(0);


            // Set pin 43 as analog input
            device.SetPinData(42, (byte)(pinFunctionsEnum.pinFunctionAnalogInput));
            Console.WriteLine("Pin 43 was set as analog input");


            int analogValue = 0;
            double analogInputVoltage;

            // Get analog input value
            device.GetAnalogInput(42, ref analogValue);

            analogInputVoltage = (double)analogValue / 1023 * 3.3; // analogValue is 10-bit number, hence 1023 used
            Console.WriteLine("\nCurrent analog input voltage: " + analogInputVoltage.ToString("0.00") + " V");

            device.DisconnectDevice();

            Console.WriteLine("\nPress any key to exit");
            Console.ReadKey();

        }
    }
}
