﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Example7_Joystick
{
    class Program
    {
        // Pin functions
        public enum pinFunctionsEnum
        {
            pinFunctionInactive = 0,
            pinFunctionDigitalInput = 2,
            pinFunctionDigitalOutput = 4,
            pinFunctionAnalogInput = 8,
            pinFunctionAnalogOutput = 16,
            pinInvert = 128
        }

        static void Main(string[] args)
        {
            Console.WriteLine("PoKeys55 Example 7 - setting joystick mapping\n");

            // Create PoKeys55 device object
            PoKeysDevice_DLL.PoKeysDevice device = new PoKeysDevice_DLL.PoKeysDevice();

            // Enumerate devices (this step must never be ommited!)
            int iNumDevices = device.EnumerateDevices();

            if (iNumDevices == 0)
            {
                Console.WriteLine("No PoKeys55 devices found!");


                Console.WriteLine("\nPress any key to exit");
                Console.ReadKey();
                return;
            }

            // Connect to first PoKeys55 device
            device.ConnectToDevice(0);


            // Map pin 43 to joystick axis x and pin 8 to joystick button 10
            
            byte[] axesMapping = new byte[6];
            byte[] buttonsMapping = new byte[32];

            Console.WriteLine("\nMapping pin 43 to X joystick axis and pin 8 to joystick button 10...");

            // First set pin as analog input
            device.SetPinData(42, (byte)pinFunctionsEnum.pinFunctionAnalogInput);
            // and pin 8 as digital input
            device.SetPinData(7, (byte)pinFunctionsEnum.pinFunctionDigitalInput);

            // Set mapping
            // Axis assignment:
            //  0 - Rx
            //  1 - Ry
            //  2 - X
            //  3 - Y
            //  4 - Z
            //  5 - Throttle
            
            axesMapping[2] = 43;      // Pin 43 to axis X
            buttonsMapping[9] = 8;    // Pin 8 to button 10 - this pin indexes are 1-based

            device.SetJoystickMapping(ref axesMapping, ref buttonsMapping);


            device.DisconnectDevice();

            Console.WriteLine("\nPress any key to exit");
            Console.ReadKey();
        }
    }
}
