#include "windows.h"
#include "conio.h"
#include "stdio.h"
#include "string.h"
#include "PoKeysLib.h"

unsigned char RequestID = 0;
WSADATA wsaData;
SOCKET comSocket;


int InitWinsock()
{
	// Initialize Winsock - version 2.2
	if (WSAStartup(MAKEWORD(2,2), &wsaData) != 0)
	{
		printf("Winsock startup failed!\n");
		return -1;
	}
	return 0;
}

int TerminateWinsock()
{
	WSACleanup();
	return 0;
}

int EnumeratePoKeysDevices(sPoKeysDevice * devicesList)
{
	SOCKET txSocket;
    sockaddr_in remoteEP;

	printf("Enumerating network PoKeys devices...\n");

    // Set up the RecvAddr structure with the broadcast ip address and correct port number
    remoteEP.sin_family = AF_INET;
    remoteEP.sin_port = htons(20055);
    remoteEP.sin_addr.s_addr = inet_addr("255.255.255.255");

    //Broadcast the message
    int t = 100; // 100 ms timeout
	int UDPbroadcast = 1;
	int status = 0;
    int BufLen = 0;
    char SendBuf[1];

    // Create socket for discovery packet
    txSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    setsockopt( txSocket, SOL_SOCKET, SO_BROADCAST, (char *)&UDPbroadcast, sizeof UDPbroadcast);
    setsockopt( txSocket, SOL_SOCKET, SO_RCVTIMEO, (char *)&t, sizeof(t));
    setsockopt( txSocket, SOL_SOCKET, SO_SNDTIMEO, (char *)&t, sizeof(t));
    status = sendto(txSocket, SendBuf, BufLen, 0, (SOCKADDR *) &remoteEP, sizeof(remoteEP));

	// Was there an error?
	if (status == SOCKET_ERROR)
	{
		printf("Failed\n");
        closesocket(txSocket);
        WSACleanup();
        return -1;
	} else
	{
		printf("Discovery packet sent!\n");
	}


	printf("Waiting for responses...\n");

  
    unsigned char rcvbuf[500];

    struct sockaddr remoteAddr;
    int remoteLen = sizeof(struct sockaddr);
	int nrOfDetectedBoards = 0;

	sPoKeysDevice * device;

	do 
    {
        // Receive response from devices
        status = recvfrom( txSocket, (char *)rcvbuf, sizeof(rcvbuf), 0, &remoteAddr, &remoteLen);

        // Get IP address and receive message
        if (status > 0)
        {
			// Parse the response
			printf("Received response...\n");

			printf("  User ID: %u\n", rcvbuf[0]);
			printf("  Serial: %u\n", (int)(256 * (int)rcvbuf[1] + rcvbuf[2]));
			printf("  Version: %u.%u\n", rcvbuf[3], rcvbuf[4]);
			printf("  IP address: %u.%u.%u.%u\n", rcvbuf[5], rcvbuf[6], rcvbuf[7], rcvbuf[8]);
			printf("  DHCP: %u\n", rcvbuf[9]);
			printf("  Host IP address: %u.%u.%u.%u\n", rcvbuf[10], rcvbuf[11], rcvbuf[12], rcvbuf[13]);             

			// Save the device info
			device = &devicesList[nrOfDetectedBoards];
			device->userID = rcvbuf[0];
			device->serialNr = (int)(256 * (int)rcvbuf[1] + rcvbuf[2]);
			device->VersionMajor = rcvbuf[3];
			device->VersionMinor = rcvbuf[4];
			memcpy(device->deviceIP, rcvbuf + 5, 4);
			device->DHCPstatus = rcvbuf[9];
			memcpy(device->hostIP, rcvbuf + 10, 4);

		    nrOfDetectedBoards++;
            status = 0;
        }
		else
		{
			if (nrOfDetectedBoards == 0)
			{
				printf("\n No Boards detected\n");
			}
		}


    } while ( status != SOCKET_ERROR && nrOfDetectedBoards < 16);

    closesocket(txSocket);
    return nrOfDetectedBoards;
}

int ConnectToPoKeysDevice(sPoKeysDevice * device)
{
    int t = 100; // 100 ms timeout

	// Create socket
	comSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

	if (comSocket == INVALID_SOCKET)
    {
        return -1; // Couldn't create the socket
    }  

	// Set socket options
	setsockopt( comSocket, SOL_SOCKET, SO_RCVTIMEO, (char *)&t, sizeof(t));
    setsockopt( comSocket, SOL_SOCKET, SO_SNDTIMEO, (char *)&t, sizeof(t));

	// Create target endpoint
    sockaddr_in remoteEP;

	// Put IP address into the string
	char addrBuf[20];
	sprintf_s(addrBuf, "%u.%u.%u.%u", device->deviceIP[0], device->deviceIP[1], device->deviceIP[2], device->deviceIP[3]);

    // Set up the RecvAddr structure with the broadcast ip address and correct port number
    remoteEP.sin_family = AF_INET;
    remoteEP.sin_port = htons(20055);
    remoteEP.sin_addr.s_addr = inet_addr(addrBuf);

	// Connect to target IP
	if (connect(comSocket, (SOCKADDR *)&remoteEP, sizeof(remoteEP)) == SOCKET_ERROR)
    {
        return -1; // Couldn't connect
    }

	printf("Connected to PoKeys device at %s", addrBuf);
	return 0;
}

int DisconnectPoKeysDevice()
{
	if (comSocket)
		closesocket(comSocket);

	return 0;
}



unsigned char CalculateChecksum(unsigned char * packet)
{
    int sum = 0;

    for (int n = 0; n < 7; n++)
    {
        sum += packet[n];
    }

    return (unsigned char)(sum % 0x100);
}

int SendRequest(unsigned char type, unsigned char data2, unsigned char data3, unsigned char data4, unsigned char data5, unsigned char* Request, unsigned char* Response)
{
	int retries1 = 0;
	int retries2 = 0;
	int result;

	unsigned char txBuffer[64];
	unsigned char rxBuffer[64];

	while (1)
	{
		// Form the request packet
		RequestID++;

		txBuffer[0] = 0xBB;
		txBuffer[1] = type;
		txBuffer[2] = data2;
		txBuffer[3] = data3;
		txBuffer[4] = data4;
		txBuffer[5] = data5;
		txBuffer[6] = RequestID;
		txBuffer[7] = CalculateChecksum(txBuffer);

		memcpy(txBuffer + 8, Request + 8, 56);

		// Send the data
		if (send(comSocket, (char *)txBuffer, 64, 0) != 64)
		{
			printf("Error sending TCP report\nAborting...\n");
			return -1;
		}

		// Wait for the response
		while(1)
		{
			result = recv(comSocket, (char *)rxBuffer, 64, 0);

			// 64 bytes received?
			if (result == 64)
			{
				if (rxBuffer[0] == 0xAA && rxBuffer[6] == RequestID)
				{
					if (rxBuffer[7] == CalculateChecksum(rxBuffer))
					{
						memcpy(Response, rxBuffer, 64);
						return 0;
					}
				}
			}
			else if (result == 0)
				printf("Connection closed\n");
			else
				printf("recv failed: %d\n", WSAGetLastError());

            
			if (++retries1 > 10) break;
		}

        if (retries2++ > 3) break;
    }

	return -1;
}



int GetDeviceType(sPoKeysDeviceStatus * deviceStat)
{
	if (deviceStat->DeviceData.SerialNumber == 0xFFFF) return 3; // old bootloader - recovery mode
	else if (deviceStat->DeviceData.SerialNumber >= 20000)
	{
		if ((deviceStat->DeviceData.FirmwareVersionMajor & (1 << 7)) > 0) return 16; // bootloader mode
		else return 11;
	}
	else if (deviceStat->DeviceData.SerialNumber >= 11500) return 2;
	else if (deviceStat->DeviceData.SerialNumber >= 10113) return 1;
	else return 0;
}



int GetDeviceData(sPoKeysDeviceStatus * deviceStat)
{
	unsigned char tempOut[64];
	unsigned char tempIn[64];

	// Get serial number and versions
    if (SendRequest(0x00, 0, 0, 0, 0, tempOut, tempIn) != 0) return -1;
	deviceStat->DeviceData.SerialNumber = (int)(tempIn[2]) * 256 + (int)tempIn[3];
	deviceStat->DeviceData.FirmwareVersionMajor = tempIn[4];
	deviceStat->DeviceData.FirmwareVersionMinor = tempIn[5];

	// Get User ID
    if (SendRequest(0x03, 0, 0, 0, 0, tempOut, tempIn) != 0) return -1;
	deviceStat->DeviceData.UserID = tempIn[2];

	// Build data
	if (SendRequest(0x04, 0, 0, 0, 0, tempOut, tempIn) != 0) return -1;
	memcpy(deviceStat->DeviceData.BuildDate + 0, tempIn + 2, 4);
	if (SendRequest(0x04, 1, 0, 0, 0, tempOut, tempIn) != 0) return -1;
	memcpy(deviceStat->DeviceData.BuildDate + 4, tempIn + 2, 4);
	if (SendRequest(0x04, 2, 0, 0, 0, tempOut, tempIn) != 0) return -1;
	memcpy(deviceStat->DeviceData.BuildDate + 8, tempIn + 2, 3);
	deviceStat->DeviceData.BuildDate[11] = 0;

	switch (GetDeviceType(deviceStat))
    {
        case 0:
			sprintf_s(deviceStat->DeviceData.DeviceTypeName, "%s", "generic PoKeys device");
			break;
        case 1:
			sprintf_s(deviceStat->DeviceData.DeviceTypeName, "%s", "PoKeys55");
			break;
        case 2:
			sprintf_s(deviceStat->DeviceData.DeviceTypeName, "%s", "PoKeys55");
			break;
        case 3:
			sprintf_s(deviceStat->DeviceData.DeviceTypeName, "%s", "PoKeys55 - recovery");
			break;
        case 10:
			sprintf_s(deviceStat->DeviceData.DeviceTypeName, "%s", "PoKeys56U");
			break;
        case 11:
			sprintf_s(deviceStat->DeviceData.DeviceTypeName, "%s", "PoKeys56E");
			break;

		case 15:
			sprintf_s(deviceStat->DeviceData.DeviceTypeName, "%s", "PoKeys56U - recovery");
			break;
        case 16:
			sprintf_s(deviceStat->DeviceData.DeviceTypeName, "%s", "PoKeys56E - recovery");
			break;

        default:
			sprintf_s(deviceStat->DeviceData.DeviceTypeName, "%s", "PoKeys");
			break;
    }

	// Device name
    if (SendRequest(0x06, 0, 0, 0, 0, tempOut, tempIn) != 0) return -1;
	memcpy(deviceStat->DeviceData.DeviceName, tempIn + 8, 10);
	deviceStat->DeviceData.DeviceName[10] = 0;

	return 0;
	
}

int SetPinFunctions(sPoKeysDeviceStatus * deviceStat)
{
	unsigned char tempOut[64];
	unsigned char tempIn[64];

	for (int i = 0; i < 55; i++)
	{
		tempOut[8 + i] = deviceStat->Pins[i].PinFunction;
	}

	// Set pin functions for all pins
    if (SendRequest(0xC0, 1, 0, 0, 0, tempOut, tempIn) != 0) return -1;

	return 0;
}

int GetDigitalInputsStatus(sPoKeysDeviceStatus * deviceStat)
{
	unsigned char tempOut[64];
	unsigned char tempIn[64];

	// Get full device report
    if (SendRequest(0xCC, 0, 0, 0, 0, tempOut, tempIn) != 0) return -1;

    for (int i = 0; i < 55; i++)
    {
		deviceStat->Pins[i].DigitalInputValue = (unsigned char)((tempIn[8 + (i / 8)] & (1 << (i % 8))) > 0 ? 1 : 0);
    }

	return 0;
}

int SetDigitalOutputsStatus(sPoKeysDeviceStatus * deviceStat)
{
	unsigned char tempOut[64] = {0};
	unsigned char tempIn[64];

    for (int i = 0; i < 55; i++)
    {
		if (deviceStat->Pins[i].DigitalOutputValue == 0)
		{
			tempOut[8 + i / 8] |= (unsigned char)(1 << (i % 8));
		}
    }
	// Set full device report
    if (SendRequest(0xCC, 1, 0, 0, 0, tempOut, tempIn) != 0) return -1;

    for (int i = 0; i < 55; i++)
    {
		deviceStat->Pins[i].DigitalInputValue = (unsigned char)((tempIn[8 + (i / 8)] & (1 << (i % 8))) > 0 ? 1 : 0);
    }

	return 0;
}

