program Example2_BasicPinFunctions;

{$APPTYPE CONSOLE}
uses
  SysUtils, PoKeysDevice_DLL_TLB;

function iif(Test: boolean; TrueR, FalseR: string): string;
begin
    if Test then
        Result := TrueR
    else
        Result := FalseR;
end;

type
   pinFunctionsEnum = (
            pinFunctionInactive = 0,
            pinFunctionDigitalInput = 2,
            pinFunctionDigitalOutput = 4,
            pinFunctionAnalogInput = 8,
            pinFunctionAnalogOutput = 16,
            pinInvert = 128
            );

  keyModifiers = (
            CtrlMask = 1,
            ShiftMask = 2,
            AltMask = 4,
            WinMask = 8,
            AltGrMask = 64
            );
var
  device: TPoKeysDevice;
  iNumDevices: Integer;
  pinState: WordBool;
begin
  try

    Writeln('PoKeys55 Example 2 - basic pin functions' + #13#10);

    // Create PoKeys55 device object
    device := TPoKeysDevice.Create(nil);
    device.Connect;

    // Enumerate devices (this step must never be ommited!)
    iNumDevices := device.EnumerateDevices();

    if iNumDevices = 0 then
    begin
        Writeln('No PoKeys55 devices found!');


        Writeln(#13#10 + 'Press any key to exit');
        Readln;

        device.Disconnect();
        device.Destroy;

        Exit;
    end;

    // Connect to first PoKeys55 device
    device.ConnectToDevice(0);

    // Set pin 1 as digital input (pin 1 has pin index 0!)
    device.SetPinData_2(0, Byte(pinFunctionsEnum.pinFunctionDigitalInput));
    Writeln('Pin 1 was set as digital input');

    // Set pin 2 as inverted digital input
    device.SetPinData_2(1, Byte(pinFunctionsEnum.pinFunctionDigitalInput) + Byte(pinFunctionsEnum.pinInvert));
    Writeln('Pin 2 was set as inverted digital input');

    // Set pin 3 as digital output
    device.SetPinData_2(2, Byte(pinFunctionsEnum.pinFunctionDigitalOutput));
    Writeln('Pin 3 was set as digital output');

    // Set pin 4 as inverted digital output
    device.SetPinData_2(3, Byte(pinFunctionsEnum.pinFunctionDigitalOutput) + Byte(pinFunctionsEnum.pinInvert));
    Writeln('Pin 4 was set as inverted digital output');

    // Set pin 5 as digital input directly mapped to a key shift+A
    device.SetPinData_2(4, Byte(pinFunctionsEnum.pinFunctionDigitalInput) + Byte(pinFunctionsEnum.pinInvert));
    device.SetPinKeyMapping(4, 4, Byte(keyModifiers.ShiftMask));

    Writeln(#13#10 + 'Reading pin states:');

    // Get pin 1 state
    device.GetInput(0, pinState);
    Writeln('Pin 1: ' + iif(pinState, 'True', 'False'));

    // Get pin 2 state
    device.GetInput(1, pinState);
    Writeln('Pin 2: ' + iif(pinState, 'True', 'False'));


    Writeln(#13#10 + 'Setting pin states:');

    // Set pin 3 state to 1
    device.SetOutput(2, true);
    Writeln('Pin 3 set to 1');

    // Set pin 4 state to 0
    device.SetOutput(3, false);
    Writeln('Pin 4 set to 0');

    // Disconnect from device
    device.DisconnectDevice();



    Writeln(#13#10 + 'Press enter key to exit');
    Readln;


    device.Disconnect();
    device.Destroy;

  except
    on E: Exception do
      Writeln(E.ClassName, ': ', E.Message);
  end;
end.
