program Example4_AnalogOutput;

{$APPTYPE CONSOLE}
uses
  SysUtils, PoKeysDevice_DLL_TLB;

function iif(Test: boolean; TrueR, FalseR: string): string;
begin
    if Test then
        Result := TrueR
    else
        Result := FalseR;
end;

type
   pinFunctionsEnum = (
            pinFunctionInactive = 0,
            pinFunctionDigitalInput = 2,
            pinFunctionDigitalOutput = 4,
            pinFunctionAnalogInput = 8,
            pinFunctionAnalogOutput = 16,
            pinInvert = 128
            );

  keyModifiers = (
            CtrlMask = 1,
            ShiftMask = 2,
            AltMask = 4,
            WinMask = 8,
            AltGrMask = 64
            );
var
  device: TPoKeysDevice;
  iNumDevices: Integer;

  analogOutputVoltage: Double;
begin
  try

    Writeln('PoKeys55 Example 4 - analog output' + #13#10);

    // Create PoKeys55 device object
    device := TPoKeysDevice.Create(nil);
    device.Connect;

    // Enumerate devices (this step must never be ommited!)
    iNumDevices := device.EnumerateDevices();

    if iNumDevices = 0 then
    begin
        Writeln('No PoKeys55 devices found!');


        Writeln(#13#10 + 'Press any key to exit');
        Readln;

        device.Disconnect();
        device.Destroy;

        Exit;
    end;

    // Connect to first PoKeys55 device
    device.ConnectToDevice(0);


    // Set pin 43 as analog output
    device.SetPinData_2(42, Byte(pinFunctionsEnum.pinFunctionAnalogOutput));
    Writeln('Pin 43 was set as analog output');

    analogOutputVoltage := 2; // 2.00 V

    // Set analog output value
    device.SetAnalogOutputValue(42, Round((analogOutputVoltage / 3.3 * 1023)));
    Writeln(#13#10 + 'Analog output voltage set to ' + Format('%2.2f',[analogOutputVoltage]) + ' V');



    // Disconnect from device
    device.DisconnectDevice();



    Writeln(#13#10 + 'Press enter key to exit');
    Readln;


    device.Disconnect();
    device.Destroy;

  except
    on E: Exception do
      Writeln(E.ClassName, ': ', E.Message);
  end;
end.

