program Example5_UsingPWMs;

{$APPTYPE CONSOLE}
uses
  SysUtils,
  PoKeysDevice_DLL_TLB,
  ActiveX,
  VarUtils,
  Variants;

var
  device: TPoKeysDevice;
  iNumDevices: Integer;

  analogOutputVoltage: Double;
  x, y, i: Byte;
  output : Variant;
  duty : Variant;
  period : Cardinal;

begin
  try

    Writeln('PoKeys55 Example 5 - using PWM' + #13#10);

    // Create PoKeys55 device object
    device := TPoKeysDevice.Create(nil);
    device.Connect;

    // Enumerate devices (this step must never be ommited!)
    iNumDevices := device.EnumerateDevices();

    if iNumDevices = 0 then
    begin
        Writeln('No PoKeys55 devices found!');


        Writeln(#13#10 + 'Press any key to exit');
        Readln;

        device.Disconnect();
        device.Destroy;

        Exit;
    end;

    // Connect to first PoKeys55 device
    device.ConnectToDevice(0);

    // All PWM channels share the same PWM frequency, set by period value
    // PoKeys55 PWM base frequency is 12 MHz = 12 000 000 ticks per second
    // To produce 1 kHz PWM frequency, period must be set to 12000 (this equals to 1 milisecond)
    // Duty cycle is a number of ticks inside period for which output will be high
    //
    //        |...............|                                               |...........
    //        |               |                                               |
    // .......|               |...............................................|
    //        0           duty cycle                                       period
    //                                                                        0
    // PWM uses special outputs mapping:
    // PWM output 0: Pin 22
    // PWM output 1: Pin 21
    // PWM output 2: Pin 20
    // PWM output 3: Pin 19
    // PWM output 4: Pin 18
    // PWM output 5: Pin 17

    output := VarArrayCreate([0, 5], varBoolean);
    duty := VarArrayCreate([0, 5], varLongWord);

    // Activate pins 17 and 18 (outputs 5 and 4)
    output[5] := true;
    output[4] := true;
    output[3] := false;
    output[2] := false;
    output[1] := false;
    output[0] := false;

    // Set period to 1 ms to produce 1 kHz PWM
    period := 12000;

    // Set duty cycle for pin 17 to 50% by setting it to half of the period
    duty[5] := 6000;

    // Set duty cycle for pin 18 to 10% by setting it to 1/10 of the period
    duty[4] := 1200;

    // Send settings to device
    device.SetPWMOutputs (PSafeArray(TVarData(output).VArray), period, PSafeArray(TVarData(duty).VArray));
    Writeln('PWM with frequency of 1 kHz on pins 17 and 18 is activated');


    Writeln(#13#10 + 'Press enter key to exit');
    Readln;


    device.Disconnect();
    device.Destroy;

  except
    on E: Exception do
      Writeln(E.ClassName, ': ', E.Message);
  end;
end.

