program Example6_UsingEncoders;

{$APPTYPE CONSOLE}
uses
  windows,
  SysUtils,
  PoKeysDevice_DLL_TLB,
  ActiveX,
  VarUtils,
  Variants;

type
   pinFunctionsEnum = (
            pinFunctionInactive = 0,
            pinFunctionDigitalInput = 2,
            pinFunctionDigitalOutput = 4,
            pinFunctionAnalogInput = 8,
            pinFunctionAnalogOutput = 16,
            pinInvert = 128
            );

    // Encoder options
    encoderOptions = (
            encoderEnabled   = 1 Shl 0,
            enable4xSampling = 1 Shl 1,
            directMappingA   = 1 Shl 4,
            macroMappingA    = 1 Shl 5,
            directMappingB   = 1 Shl 6,
            macroMappingB    = 1 Shl 7
            );

    keyModifiers = (
            CtrlMask = 1,
            ShiftMask = 2,
            AltMask = 4,
            WinMask = 8,
            AltGrMask = 64
            );

    // Key codes
    ekeyCodes = (
            key_a = $4, // Shift combination: A
            key_b = $5, // Shift combination: B
            key_c = $6, // Shift combination: C
            key_d = $7, // Shift combination: D
            key_e = $8, // Shift combination: E
            key_f = $9, // Shift combination: F
            key_g = $0A, // Shift combination: G
            key_h = $0B, // Shift combination: H
            key_i = $0C, // Shift combination: I
            key_j = $0D, // Shift combination: J
            key_k = $0E, // Shift combination: K
            key_l = $0F, // Shift combination: L
            key_m = $10, // Shift combination: M
            key_n = $11, // Shift combination: N
            key_o = $12, // Shift combination: O
            key_p = $13, // Shift combination: P
            key_q = $14, // Shift combination: Q
            key_r = $15, // Shift combination: R
            key_s = $16, // Shift combination: S
            key_t = $17, // Shift combination: T
            key_u = $18, // Shift combination: U
            key_v = $19, // Shift combination: V
            key_w = $1A, // Shift combination: W
            key_x = $1B, // Shift combination: X
            key_y = $1C, // Shift combination: Y
            key_z = $1D, // Shift combination: Z
            key_1 = $1E, // Shift combination: !
            key_2 = $1F, // Shift combination: @
            key_3 = $20, // Shift combination: #
            key_4 = $21, // Shift combination: $
            key_5 = $22, // Shift combination: %
            key_6 = $23, // Shift combination: ^
            key_7 = $24, // Shift combination: &
            key_8 = $25, // Shift combination: *
            key_9 = $26, // Shift combination: (
            key_0 = $27, // Shift combination: )
            key_Minus = $2D, // Shift combination: _
            key_Equal = $2E, // Shift combination: +
            key_SquareOpeningBracket = $2F, // Shift combination: {
            key_SquareClosingBracket = $30, // Shift combination: }
            key_BackSlash = $31, // Shift combination: |
            key_Semicolon = $33, // Shift combination: :
            key_Apostrophe = $34, // Shift combination:
            key_Tilde = $35, // Shift combination: ~
            key_Comma = $36, // Shift combination: <
            key_FullStop = $37, // Shift combination: >
            key_Slash = $38, // Shift combination: ?
            key_Return = $28, // Shift combination:
            key_Escape = $29, // Shift combination:
            key_Backspace = $2A, // Shift combination:
            key_Tab = $2B, // Shift combination:
            key_Space = $2C, // Shift combination:
            key_CapsLock = $39, // Shift combination:
            key_F1 = $3A, // Shift combination:
            key_F2 = $3B, // Shift combination:
            key_F3 = $3C, // Shift combination:
            key_F4 = $3D, // Shift combination:
            key_F5 = $3E, // Shift combination:
            key_F6 = $3F, // Shift combination:
            key_F7 = $40, // Shift combination:
            key_F8 = $41, // Shift combination:
            key_F9 = $42, // Shift combination:
            key_F10 = $43, // Shift combination:
            key_F11 = $44, // Shift combination:
            key_F12 = $45, // Shift combination:
            key_PrintScreen = $46, // Shift combination:
            key_ScrollLock = $47, // Shift combination:
            key_Insert = $49, // Shift combination:
            key_Home = $4A, // Shift combination:
            key_PageUp = $4B, // Shift combination:
            key_Delete = $4C, // Shift combination:
            key_End = $4D, // Shift combination:
            key_PageDown = $4E, // Shift combination:
            key_RightArrow = $4F, // Shift combination:
            key_LeftArrow = $50, // Shift combination:
            key_DownArrow = $51, // Shift combination:
            key_UpArrow = $52, // Shift combination:
            key_NumLock = $53, // Shift combination:
            key_KeypadDivide = $54, // Shift combination:
            key_KeypadMultiply = $55, // Shift combination:
            key_KeypadMinus = $56, // Shift combination:
            key_KeypadPlus = $57, // Shift combination:
            key_KeypadEnter = $58, // Shift combination:
            key_Keypad1 = $59, // Shift combination:
            key_Keypad2 = $5A, // Shift combination:
            key_Keypad3 = $5B, // Shift combination:
            key_Keypad4 = $5C, // Shift combination:
            key_Keypad5 = $5D, // Shift combination:
            key_Keypad6 = $5E, // Shift combination:
            key_Keypad7 = $5F, // Shift combination:
            key_Keypad8 = $60, // Shift combination:
            key_Keypad9 = $61, // Shift combination:
            key_Keypad0 = $62, // Shift combination:
            key_KeypadDot = $63 // Shift combination:
        );
var
  device: TPoKeysDevice;
  iNumDevices: Integer;

  encoder1Value, encoder2Value, encoder3Value : Integer;

  Cin,Cout:Thandle;

  function getstdhandles:boolean;
    begin
      cin:=getstdhandle(STD_INPUT_HANDLE);
      cout:=getstdhandle(STD_OUTPUT_HANDLE);
      result:= (cin <> INVALID_HANDLE_VALUE ) and  (cin <> INVALID_HANDLE_VALUE );
    end;

  function keypressed:boolean;
    var inputrecords:DWORD;
    var ir:_INPUT_RECORD;
    var bufcount:DWORD;

    begin
      inputrecords:=0;
      GetNumberOfConsoleInputEvents(cin,inputrecords);
      result:=false;
      if inputrecords<> 0 then
      begin
        peekConsoleinput( cin,ir,1,bufcount);
        if bufcount <> 0 then
        begin
          if ir.EventType = KEY_EVENT then
          begin
            if ir.Event.KeyEvent.bKeyDown then
              result:=true
            else
              flushconsoleinputbuffer(cin);
          end
          else
            flushconsoleinputbuffer(cin);
        end;
      end
    end;


begin
  try

    Writeln('PoKeys55 Example 6 - using encoders' + #13#10);

    // Create PoKeys55 device object
    device := TPoKeysDevice.Create(nil);
    device.Connect;

    // Enumerate devices (this step must never be ommited!)
    iNumDevices := device.EnumerateDevices();

    if iNumDevices = 0 then
    begin
        Writeln('No PoKeys55 devices found!');


        Writeln(#13#10 + 'Press any key to exit');
        Readln;

        device.Disconnect();
        device.Destroy;

        Exit;
    end;

    // Connect to first PoKeys55 device
    device.ConnectToDevice(0);


    Writeln(#13#10 + 'Setting encoder 1 to use pins 1 and 2...');
    // Let's configure encoder 1 to use pins 1 (channel A) and 2 (channel B) with no key mappings
    // Pins 1 and 2 must be set as digital inputs at first
    device.SetPinData_2(0, Byte(pinFunctionsEnum.pinFunctionDigitalInput));
    device.SetPinData_2(1, Byte(pinFunctionsEnum.pinFunctionDigitalInput));
    device.SetEncoderSettings(0, Byte(encoderOptions.encoderEnabled), 0, 1);

    Writeln('Setting encoder 2 to use pins 3 and 4...');
    // and encoder 2 to use pins 3 (channel A) and 4 (channel B) with direct key mapping
    // Pins 3 and 4 must also be set as digital inputs at first
    device.SetPinData_2(2, Byte(pinFunctionsEnum.pinFunctionDigitalInput));
    device.SetPinData_2(3, Byte(pinFunctionsEnum.pinFunctionDigitalInput));
    device.SetEncoderSettings(1, Byte(encoderOptions.encoderEnabled) +
                                 Byte(encoderOptions.directMappingA) +
                                 Byte(encoderOptions.directMappingB), 2, 3);
    // Let direction A be mapped to up key and direction B to shift + down key
    device.SetEncoderKeyMappingDirA_2(1, Byte(ekeyCodes.key_a), 0);
    device.SetEncoderKeyMappingDirB_2(1, Byte(ekeyCodes.key_b), Byte(keyModifiers.ShiftMask));

    Writeln('Setting encoder 3 to use pins 5 and 6...');
    // and encoder 3 to use pins 5 (channel A) and 6 (channel B) in 4x sampling mode
    // Pins 5 and 6 must also be set as digital inputs at first
    device.SetPinData_2(4, Byte(pinFunctionsEnum.pinFunctionDigitalInput));
    device.SetPinData_2(5, Byte(pinFunctionsEnum.pinFunctionDigitalInput));
    device.SetEncoderSettings(2, Byte(encoderOptions.encoderEnabled) +
                                 Byte(encoderOptions.enable4xSampling), 4, 5);

    Writeln(#13#10 + 'Entering display loop... press Escape to quit');

    // Reset encoder 1 first
    device.ResetEncoderValue(0);

    getstdhandles;

    // Now let's create a loop and read the raw values of the encoders
    while (keypressed = false) do
    begin

        // Get RAW values from device
        device.GetEncoderRAWValue(0, encoder1Value);
        device.GetEncoderRAWValue(1, encoder2Value);
        device.GetEncoderRAWValue(2, encoder3Value);

        Writeln(#13 + 'Encoder 1 value: ' + IntToStr(encoder1Value) +
                  '    Encoder 2 value: ' + IntToStr(encoder2Value) +
                  '    Encoder 3 value: ' + IntToStr(encoder3Value) + '       ');
    end;


    Writeln(#13#10 + 'Press enter key to exit');
    Readln;


    device.Disconnect();
    device.Destroy;

  except
    on E: Exception do
      Writeln(E.ClassName, ': ', E.Message);
  end;
end.

