program Example7_Joystick;

{$APPTYPE CONSOLE}
uses
  windows,
  SysUtils,
  PoKeysDevice_DLL_TLB,
  ActiveX,
  VarUtils,
  Variants;

type
   pinFunctionsEnum = (
            pinFunctionInactive = 0,
            pinFunctionDigitalInput = 2,
            pinFunctionDigitalOutput = 4,
            pinFunctionAnalogInput = 8,
            pinFunctionAnalogOutput = 16,
            pinInvert = 128
            );

var
  device: TPoKeysDevice;
  iNumDevices: Integer;

  axesMapping, buttonsMapping : Variant;

begin
  try

    Writeln('PoKeys55 Example 7 - setting joystick mapping' + #13#10);

    // Create PoKeys55 device object
    device := TPoKeysDevice.Create(nil);
    device.Connect;

    // Enumerate devices (this step must never be ommited!)
    iNumDevices := device.EnumerateDevices();

    if iNumDevices = 0 then
    begin
        Writeln('No PoKeys55 devices found!');


        Writeln(#13#10 + 'Press any key to exit');
        Readln;

        device.Disconnect();
        device.Destroy;

        Exit;
    end;

    // Connect to first PoKeys55 device
    device.ConnectToDevice(0);


    // Map pin 43 to joystick axis x and pin 8 to joystick button 10
    axesMapping := VarArrayCreate([0, 5], varByte);
    buttonsMapping := VarArrayCreate([0, 31], varByte);

    Writeln(#13#10 + 'Mapping pin 43 to X joystick axis and pin 8 to joystick button 10...');

    // First set pin as analog input
    device.SetPinData_2(42, Byte(pinFunctionsEnum.pinFunctionAnalogInput));
    // and pin 8 as digital input
    device.SetPinData_2(7, Byte(pinFunctionsEnum.pinFunctionDigitalInput));

    // Set mapping
    // Axis assignment:
    //  0 - Rx
    //  1 - Ry
    //  2 - X
    //  3 - Y
    //  4 - Z
    //  5 - Throttle

    axesMapping[2] := 43;      // Pin 43 to axis X
    buttonsMapping[9] := 8;    // Pin 8 to button 10 - this pin indexes are 1-based

    device.SetJoystickMapping(PSafeArray(TVarData(axesMapping).VArray), PSafeArray(TVarData(buttonsMapping).VArray));

    Writeln(#13#10 + 'Press enter key to exit');
    Readln;


    device.Disconnect();
    device.Destroy;

  except
    on E: Exception do
      Writeln(E.ClassName, ': ', E.Message);
  end;
end.

