program Example8_MatrixLED;

{$APPTYPE CONSOLE}
uses
  SysUtils,
  PoKeysDevice_DLL_TLB, ActiveX, VarUtils, Variants;

var
  device: TPoKeysDevice;
  iNumDevices: Integer;

  analogOutputVoltage: Double;
  x, y, i: Byte;
  displayBuffer : Variant;

begin
  try

    Writeln('PoKeys55 Example 8 - matrix LED' + #13#10);

    // Create PoKeys55 device object
    device := TPoKeysDevice.Create(nil);
    device.Connect;

    // Enumerate devices (this step must never be ommited!)
    iNumDevices := device.EnumerateDevices();

    if iNumDevices = 0 then
    begin
        Writeln('No PoKeys55 devices found!');


        Writeln(#13#10 + 'Press any key to exit');
        Readln;

        device.Disconnect();
        device.Destroy;

        Exit;
    end;

    // Connect to first PoKeys55 device
    device.ConnectToDevice(0);


    // Connect matrix LED display accordingly to the schematics found in PoKeys55 user manual
    // Use this pin mapping:
    //   - pin 9 as serial data
    //   - pin 10 as shift register latch clock
    //   - pin 11 as serial clock

    // Use only display 1
    device.MatrixLEDSetSettings(true, 5, 5, false, 0, 0);

    // Fill the display pixel by pixel
    device.MatrixLED1ClearAll(false);

    for y:=0 to 4 do
    begin
        for x:=0 to 4 do
        begin
            device.MatrixLED1SetPixel(y, x, true);
            Sleep(20);
        end;
    end;

    // Then clear the whole display pixel by pixel
    for y:=0 to 4 do
    begin
        for x:=0 to 4 do
        begin
            device.MatrixLED1SetPixel(y, x, false);
            Sleep(20);
        end;
    end;

    // Update display with some data (increasing bars)
    // these bytes are pixel mapped
    // displayBuffer[0].0 (first bit in first byte) represents first pixel on the display
    displayBuffer := VarArrayCreate([0, 7], varByte);
    displayBuffer[0] := 1;
    displayBuffer[1] := 3;
    displayBuffer[2] := 7;
    displayBuffer[3] := 15;
    displayBuffer[4] := 31;
    displayBuffer[5] := 0;
    displayBuffer[6] := 0;
    displayBuffer[7] := 0;

    device.MatrixLED1Update(PSafeArray(TVarData(displayBuffer).VArray));

    // Disconnect from device
    device.DisconnectDevice();



    Writeln(#13#10 + 'Press enter key to exit');
    Readln;


    device.Disconnect();
    device.Destroy;

  except
    on E: Exception do
      Writeln(E.ClassName, ': ', E.Message);
  end;
end.

