program Example9_MatrixKeyboard;

{$APPTYPE CONSOLE}
uses
  windows,
  SysUtils,
  PoKeysDevice_DLL_TLB,
  ActiveX,
  VarUtils,
  Variants;

type
   pinFunctionsEnum = (
            pinFunctionInactive = 0,
            pinFunctionDigitalInput = 2,
            pinFunctionDigitalOutput = 4,
            pinFunctionAnalogInput = 8,
            pinFunctionAnalogOutput = 16,
            pinInvert = 128
            );

    // Encoder options
    encoderOptions = (
            encoderEnabled   = 1 Shl 0,
            enable4xSampling = 1 Shl 1,
            directMappingA   = 1 Shl 4,
            macroMappingA    = 1 Shl 5,
            directMappingB   = 1 Shl 6,
            macroMappingB    = 1 Shl 7
            );

    ekeyModifiers = (
            CtrlMask = 1,
            ShiftMask = 2,
            AltMask = 4,
            WinMask = 8,
            AltGrMask = 64
            );

    // Key codes
    ekeyCodes = (
            key_a = $4, // Shift combination: A
            key_b = $5, // Shift combination: B
            key_c = $6, // Shift combination: C
            key_d = $7, // Shift combination: D
            key_e = $8, // Shift combination: E
            key_f = $9, // Shift combination: F
            key_g = $0A, // Shift combination: G
            key_h = $0B, // Shift combination: H
            key_i = $0C, // Shift combination: I
            key_j = $0D, // Shift combination: J
            key_k = $0E, // Shift combination: K
            key_l = $0F, // Shift combination: L
            key_m = $10, // Shift combination: M
            key_n = $11, // Shift combination: N
            key_o = $12, // Shift combination: O
            key_p = $13, // Shift combination: P
            key_q = $14, // Shift combination: Q
            key_r = $15, // Shift combination: R
            key_s = $16, // Shift combination: S
            key_t = $17, // Shift combination: T
            key_u = $18, // Shift combination: U
            key_v = $19, // Shift combination: V
            key_w = $1A, // Shift combination: W
            key_x = $1B, // Shift combination: X
            key_y = $1C, // Shift combination: Y
            key_z = $1D, // Shift combination: Z
            key_1 = $1E, // Shift combination: !
            key_2 = $1F, // Shift combination: @
            key_3 = $20, // Shift combination: #
            key_4 = $21, // Shift combination: $
            key_5 = $22, // Shift combination: %
            key_6 = $23, // Shift combination: ^
            key_7 = $24, // Shift combination: &
            key_8 = $25, // Shift combination: *
            key_9 = $26, // Shift combination: (
            key_0 = $27, // Shift combination: )
            key_Minus = $2D, // Shift combination: _
            key_Equal = $2E, // Shift combination: +
            key_SquareOpeningBracket = $2F, // Shift combination: {
            key_SquareClosingBracket = $30, // Shift combination: }
            key_BackSlash = $31, // Shift combination: |
            key_Semicolon = $33, // Shift combination: :
            key_Apostrophe = $34, // Shift combination:
            key_Tilde = $35, // Shift combination: ~
            key_Comma = $36, // Shift combination: <
            key_FullStop = $37, // Shift combination: >
            key_Slash = $38, // Shift combination: ?
            key_Return = $28, // Shift combination:
            key_Escape = $29, // Shift combination:
            key_Backspace = $2A, // Shift combination:
            key_Tab = $2B, // Shift combination:
            key_Space = $2C, // Shift combination:
            key_CapsLock = $39, // Shift combination:
            key_F1 = $3A, // Shift combination:
            key_F2 = $3B, // Shift combination:
            key_F3 = $3C, // Shift combination:
            key_F4 = $3D, // Shift combination:
            key_F5 = $3E, // Shift combination:
            key_F6 = $3F, // Shift combination:
            key_F7 = $40, // Shift combination:
            key_F8 = $41, // Shift combination:
            key_F9 = $42, // Shift combination:
            key_F10 = $43, // Shift combination:
            key_F11 = $44, // Shift combination:
            key_F12 = $45, // Shift combination:
            key_PrintScreen = $46, // Shift combination:
            key_ScrollLock = $47, // Shift combination:
            key_Insert = $49, // Shift combination:
            key_Home = $4A, // Shift combination:
            key_PageUp = $4B, // Shift combination:
            key_Delete = $4C, // Shift combination:
            key_End = $4D, // Shift combination:
            key_PageDown = $4E, // Shift combination:
            key_RightArrow = $4F, // Shift combination:
            key_LeftArrow = $50, // Shift combination:
            key_DownArrow = $51, // Shift combination:
            key_UpArrow = $52, // Shift combination:
            key_NumLock = $53, // Shift combination:
            key_KeypadDivide = $54, // Shift combination:
            key_KeypadMultiply = $55, // Shift combination:
            key_KeypadMinus = $56, // Shift combination:
            key_KeypadPlus = $57, // Shift combination:
            key_KeypadEnter = $58, // Shift combination:
            key_Keypad1 = $59, // Shift combination:
            key_Keypad2 = $5A, // Shift combination:
            key_Keypad3 = $5B, // Shift combination:
            key_Keypad4 = $5C, // Shift combination:
            key_Keypad5 = $5D, // Shift combination:
            key_Keypad6 = $5E, // Shift combination:
            key_Keypad7 = $5F, // Shift combination:
            key_Keypad8 = $60, // Shift combination:
            key_Keypad9 = $61, // Shift combination:
            key_Keypad0 = $62, // Shift combination:
            key_KeypadDot = $63 // Shift combination:
        );
var
  device: TPoKeysDevice;
  iNumDevices: Integer;

  matrixKeyboardSetting, rows, cols : Byte;
  row_pins, col_pins, macro_mapping, key_codes, key_modifiers : Variant;

  i : Integer;

begin
  try

    Writeln('PoKeys55 Example 9 - matrix keyboard' + #13#10);

    // Create PoKeys55 device object
    device := TPoKeysDevice.Create(nil);
    device.Connect;

    // Enumerate devices (this step must never be ommited!)
    iNumDevices := device.EnumerateDevices();

    if iNumDevices = 0 then
    begin
        Writeln('No PoKeys55 devices found!');


        Writeln(#13#10 + 'Press any key to exit');
        Readln;

        device.Disconnect();
        device.Destroy;

        Exit;
    end;

    // Connect to first PoKeys55 device
    device.ConnectToDevice(0);



    matrixKeyboardSetting := 1;  // enabled
    rows := 4;                   // 4 rows
    cols := 4;                   // 4 columns

    row_pins := VarArrayCreate([0, 15], varByte);
    col_pins := VarArrayCreate([0, 7], varByte);
    macro_mapping := VarArrayCreate([0, 127], varBoolean);
    key_codes := VarArrayCreate([0, 127], varByte);
    key_modifiers := VarArrayCreate([0, 127], varByte);

    // Set remaining pins to 0xFF
    for i := 0 to 7 do
    begin
        row_pins[i] := $FF;
        col_pins[i] := $FF;
    end;

    // PoKeys55 pins 1 to 4 as matrix keyboard columns (digital inputs)
    col_pins[0] := 0;
    col_pins[1] := 1;
    col_pins[2] := 2;
    col_pins[3] := 3;

    device.SetPinData_2(0, Byte(pinFunctionsEnum.pinFunctionDigitalInput));
    device.SetPinData_2(1, Byte(pinFunctionsEnum.pinFunctionDigitalInput));
    device.SetPinData_2(2, Byte(pinFunctionsEnum.pinFunctionDigitalInput));
    device.SetPinData_2(3, Byte(pinFunctionsEnum.pinFunctionDigitalInput));

    // PoKeys55 pins 5 to 8 as matrix keyboard rows (digital outputs)
    row_pins[0] := 4;
    row_pins[1] := 5;
    row_pins[2] := 6;
    row_pins[3] := 7;

    device.SetPinData_2(4, Byte(pinFunctionsEnum.pinFunctionDigitalOutput));
    device.SetPinData_2(5, Byte(pinFunctionsEnum.pinFunctionDigitalOutput));
    device.SetPinData_2(6, Byte(pinFunctionsEnum.pinFunctionDigitalOutput));
    device.SetPinData_2(7, Byte(pinFunctionsEnum.pinFunctionDigitalOutput));

    // Use standard 4x4 matrix keyboard mapping
    key_codes[0] := Byte(ekeyCodes.key_1);
    key_codes[1] := Byte(ekeyCodes.key_2);
    key_codes[2] := Byte(ekeyCodes.key_3);
    key_codes[3] := Byte(ekeyCodes.key_a);

    key_codes[8] := Byte(ekeyCodes.key_4);
    key_codes[9] := Byte(ekeyCodes.key_5);
    key_codes[10] := Byte(ekeyCodes.key_6);
    key_codes[11] := Byte(ekeyCodes.key_b);

    key_codes[16] := Byte(ekeyCodes.key_7);
    key_codes[17] := Byte(ekeyCodes.key_8);
    key_codes[18] := Byte(ekeyCodes.key_9);
    key_codes[19] := Byte(ekeyCodes.key_c);

    key_codes[24] := Byte(ekeyCodes.key_KeypadMultiply);
    key_codes[25] := Byte(ekeyCodes.key_0);
    key_codes[26] := Byte(ekeyCodes.key_3);
    key_codes[27] := Byte(ekeyCodes.key_d);

    key_modifiers[26] := Byte(ekeyModifiers.ShiftMask);

    device.SetMatrixKeyboardConfiguration(matrixKeyboardSetting, cols, rows, PSafeArray(TVarData(row_pins).VArray),
                    PSafeArray(TVarData(col_pins).VArray), PSafeArray(TVarData(macro_mapping).VArray),
                    PSafeArray(TVarData(key_codes).VArray), PSafeArray(TVarData(key_modifiers).VArray));

    Writeln('Matrix keyboard configured as standard 4x4');



    Writeln(#13#10 + 'Press enter key to exit');
    Readln;


    device.Disconnect();
    device.Destroy;

  except
    on E: Exception do
      Writeln(E.ClassName, ': ', E.Message);
  end;
end.

