﻿Module Module1

    ' Encoder options
    Public Enum encoderOptions
        encoderEnabled = 1 << 0
        enable4xSampling = 1 << 1
        directMappingA = 1 << 4
        macroMappingA = 1 << 5
        directMappingB = 1 << 6
        macroMappingB = 1 << 7
    End Enum

    ' Key modifiers
    Public Enum keyModifiers
        CtrlMask = 1
        ShiftMask = 2
        AltMask = 4
        WinMask = 8
        AltGrMask = 64
    End Enum

    ' Pin functions
    Public Enum pinFunctionsEnum
        pinFunctionInactive = 0
        pinFunctionDigitalInput = 2
        pinFunctionDigitalOutput = 4
        pinFunctionAnalogInput = 8
        pinFunctionAnalogOutput = 16
        pinInvert = 128
    End Enum

    Sub Main()
        Console.WriteLine("PoKeys55 Example 6 - using encoder switches" & vbLf)

        ' Create PoKeys55 device object
        Dim device As New PoKeysDevice_DLL.PoKeysDevice()

        ' Enumerate devices (this step must never be ommited!)
        Dim iNumDevices As Integer = device.EnumerateDevices()

        If iNumDevices = 0 Then
            Console.WriteLine("No PoKeys55 devices found!")


            Console.WriteLine(vbLf & "Press any key to exit")
            Console.ReadKey()
            Exit Sub
        End If

        ' Connect to first PoKeys55 device
        device.ConnectToDevice(0)

        Console.WriteLine(vbLf & "Setting encoder 1 to use pins 1 and 2...")
        ' Let's configure encoder 1 to use pins 1 (channel A) and 2 (channel B) with no key mappings
        ' Pins 1 and 2 must be set as digital inputs at first
        device.SetPinData(0, CByte(pinFunctionsEnum.pinFunctionDigitalInput))
        device.SetPinData(1, CByte(pinFunctionsEnum.pinFunctionDigitalInput))
        device.SetEncoderSettings(0, CByte(encoderOptions.encoderEnabled), 0, 1)

        Console.WriteLine("Setting encoder 2 to use pins 3 and 4...")
        ' and encoder 2 to use pins 3 (channel A) and 4 (channel B) with direct key mapping
        ' Pins 3 and 4 must also be set as digital inputs at first
        device.SetPinData(2, CByte(pinFunctionsEnum.pinFunctionDigitalInput))
        device.SetPinData(3, CByte(pinFunctionsEnum.pinFunctionDigitalInput))
        device.SetEncoderSettings(1, CByte(encoderOptions.encoderEnabled) + CByte(encoderOptions.directMappingA) + CByte(encoderOptions.directMappingB), 2, 3)
        ' Let direction A be mapped to up key and direction B to shift + down key
        device.SetEncoderKeyMappingDirA(1, &H52, 0)
        device.SetEncoderKeyMappingDirB(1, &H51, CByte(keyModifiers.ShiftMask))

        Console.WriteLine("Setting encoder 3 to use pins 5 and 6...")
        ' and encoder 3 to use pins 5 (channel A) and 6 (channel B) in 4x sampling mode
        ' Pins 5 and 6 must also be set as digital inputs at first
        device.SetPinData(4, CByte(pinFunctionsEnum.pinFunctionDigitalInput))
        device.SetPinData(5, CByte(pinFunctionsEnum.pinFunctionDigitalInput))
        device.SetEncoderSettings(2, CByte(encoderOptions.encoderEnabled) + CByte(encoderOptions.enable4xSampling), 4, 5)

        Console.WriteLine(vbLf & "Entering display loop... press Escape to quit")

        ' Now let's create a loop and read the raw values of the encoders

        ' Reset encoder 1 first
        device.ResetEncoderValue(0)

        Dim cki As ConsoleKeyInfo
        Dim encoder1Value As Integer = 0
        Dim encoder2Value As Integer = 0
        Dim encoder3Value As Integer = 0

        While True
            If Console.KeyAvailable = True Then
                cki = Console.ReadKey()

                If cki.Key = ConsoleKey.Escape Then
                    Exit While
                End If
            End If

            ' Get RAW values from device
            device.GetEncoderRAWValue(0, encoder1Value)
            device.GetEncoderRAWValue(1, encoder2Value)
            device.GetEncoderRAWValue(2, encoder3Value)

            Console.Write(((vbCr & "Encoder 1 value: " & encoder1Value.ToString & _
                            "    Encoder 2 value: ") + encoder2Value.ToString & _
                            "    Encoder 3 value: ") + encoder3Value.ToString & "       ")
        End While

        device.DisconnectDevice()

        Console.WriteLine(vbLf & "Press any key to exit")
        Console.ReadKey()
    End Sub

End Module
