﻿Module Module1

    ' Pin functions
    Public Enum pinFunctionsEnum
        pinFunctionInactive = 0
        pinFunctionDigitalInput = 2
        pinFunctionDigitalOutput = 4
        pinFunctionAnalogInput = 8
        pinFunctionAnalogOutput = 16
        pinInvert = 128
    End Enum

    Sub Main()
        Console.WriteLine("PoKeys55 Example 7 - setting joystick mapping" & vbLf)

        ' Create PoKeys55 device object
        Dim device As New PoKeysDevice_DLL.PoKeysDevice()

        ' Enumerate devices (this step must never be ommited!)
        Dim iNumDevices As Integer = device.EnumerateDevices()

        If iNumDevices = 0 Then
            Console.WriteLine("No PoKeys55 devices found!")


            Console.WriteLine(vbLf & "Press any key to exit")
            Console.ReadKey()
            Exit Sub
        End If

        ' Connect to first PoKeys55 device
        device.ConnectToDevice(0)


        ' Map pin 43 to joystick axis x and pin 8 to joystick button 10

        Dim axesMapping As Byte() = New Byte(5) {}
        Dim buttonsMapping As Byte() = New Byte(31) {}

        Console.WriteLine(vbLf & "Mapping pin 43 to X joystick axis and pin 8 to joystick button 10...")

        ' First set pin as analog input
        device.SetPinData(42, CByte(pinFunctionsEnum.pinFunctionAnalogInput))
        ' and pin 8 as digital input
        device.SetPinData(7, CByte(pinFunctionsEnum.pinFunctionDigitalInput))

        ' Set mapping
        ' Axis assignment:
        '  0 - Rx
        '  1 - Ry
        '  2 - X
        '  3 - Y
        '  4 - Z
        '  5 - Throttle

        axesMapping(2) = 43
        ' Pin 43 to axis X
        buttonsMapping(9) = 8
        ' Pin 8 to button 10 - this pin indexes are 1-based
        device.SetJoystickMapping(axesMapping, buttonsMapping)


        device.DisconnectDevice()

        Console.WriteLine(vbLf & "Press any key to exit")
        Console.ReadKey()
    End Sub


End Module
